﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace CSHowDoISamples
{
    public partial class AddAnInMemoryMarkerOverlay : UserControl
    {
        public AddAnInMemoryMarkerOverlay()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-155.733, 95.60, 104.42, -81.9);

            WorldMapKitWmsWpfOverlay worldMapKitOverlay = new WorldMapKitWmsWpfOverlay();
            wpfMap1.Overlays.Add(worldMapKitOverlay);

            InMemoryMarkerOverlay markerOverlay = new InMemoryMarkerOverlay();
            markerOverlay.Columns.Add(new FeatureSourceColumn("Name"));
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ImageSource = new BitmapImage(new Uri("/Resources/AQUA.png", UriKind.RelativeOrAbsolute));
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Width = 20;
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.Height = 34;
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.YOffset = -17;
            markerOverlay.ZoomLevelSet.ZoomLevel01.DefaultPointMarkerStyle.ToolTip = "This is [#Name#].";
            markerOverlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            wpfMap1.Overlays.Add("MarkerOverlay", markerOverlay);

            Feature newFeature = new Feature(-95.2806, 38.9554);
            newFeature.ColumnValues.Add("Name", "Lawrence");

            markerOverlay.FeatureSource.BeginTransaction();
            markerOverlay.FeatureSource.AddFeature(newFeature);
            markerOverlay.FeatureSource.CommitTransaction();

            wpfMap1.Refresh();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // Solution 1
            //wpfMap1.GetBitmap((int)wpfMap1.ActualWidth, (int)wpfMap1.ActualHeight).Save(@"D:\test2.bmp");

            // Solution 2
            ControlToBitmap.SaveToBitMapOfControl(wpfMap1, @"D:\test.bmp");
        }
    }

    internal static class ControlToBitmap
    {
        private static BitmapEncoder GetImageFromControl(FrameworkElement targetControl)
        {
            var renderBitmap = new RenderTargetBitmap((Int32)targetControl.ActualWidth, (Int32)targetControl.ActualHeight, 96d,
                                                      96d, PixelFormats.Pbgra32);
            renderBitmap.Render(targetControl);

            BitmapEncoder encoder = new BmpBitmapEncoder();
            
            encoder.Frames.Add(BitmapFrame.Create(renderBitmap));

            return encoder;
        }
                
        public static Boolean SaveToBitMapOfControl(FrameworkElement controlToConvert, String fileName)
        {
            try
            {
                using (var outStream = new FileStream(fileName, FileMode.Create))
                {
                    var encoder = GetImageFromControl(controlToConvert);
                    encoder.Save(outStream);
                }
            }
            catch (Exception ex)
            {
                return false;
            } 

            return true;
        }

    }
}
